######################################## 
# Bayesian CFA of the MSPSS            #
# Pedro Henrique Ribeiro Santiago      #
# Adrian Quintero                      #
# Dandara Haag                         #
# Rachel Roberts                       #
# Lisa Smithers                        #
# Lisa Jamieson                        #
# 02-06-2019                           #
########################################

library(foreign)
library(plyr)
library(psych)
library(lavaan)
library(blavaan)
library(semTools)
library(simsem)
library(coda)
library(rstan)
library(ggplot2)
library(HDInterval)
library(mcmcse)
library(Rmpfr)
library(scales)
library(gridExtra)
library(epiDisplay)
library(bayesplot)
library(Bayesrel)

### DESCRIPTIVE STATISTICS ###

tab1(NSAOHfull$Sex, sort.group = "increasing", cum.percent = TRUE)
tab1(NSAOHfull$Education, sort.group = "increasing", cum.percent = TRUE)
psych::describe(dataset2$AGE, skew=FALSE)
tab1(dataset2$employ_dichot, sort.group = "increasing", cum.percent = TRUE)

### INVESTIGATE MISSINGNESS ###

propmis <- c((length(NSAOHfull$M1[is.na(NSAOHfull$M1)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M1)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M2)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M2)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M5)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M5)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M10)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M10)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M3)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M3)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M4)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M4)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M8)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M8)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M11)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M11)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M6)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M6)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M7)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M7)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M9)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M9)==FALSE]))*100, 
             (length(NSAOHfull$M1[is.na(NSAOHfull$M12)==TRUE])/length(NSAOHfull$M1[is.na(NSAOHfull$M12)==FALSE]))*100) 
min(propmis)
max(propmis)

### COMPlETE CASE ANALYSIS ###

NSAOHfull <- NSAOHfull[,6:17]
NSAOHfull <- NSAOHfull[complete.cases(NSAOHfull),]

### CREATE THE FACTORIAL STRUCTURES ###

MSSSone <- '
one =~ M1 + M2 + M5 + M10 + M3 + M4 + M8 + M11 + M6 + M7 + M9 + M12'

MSSStwo <- '
one =~ M1 + M2 + M5 + M10 + M6 + M7 + M9 + M12
two =~ M3 + M4 + M8 + M11'

MSSSthree <- '
one =~ M1 + M2 + M5 + M10
two =~ M3 + M4 + M8 + M11 
three =~ M6 + M7 + M9 + M12'

#############################################################

### BAYESIAN INFERENCE ###

## fit null model to calculate CFI, TLI, and NFI
null.model <- c(paste0("M", 1:12, " ~~ M", 1:12), paste0("M", 1:12, " ~ 1"))
fit0 <- bcfa(null.model, data = NSAOHfull, cp = "srs",
             n.chains = 3, burnin = 1000, sample = 1000)
summary(fit0)
bnull <- blavFitIndices(fit0, baseline.model = fit0, rescale="devM", pD="dic")

## one-factor model ##

fit1 <- bcfa(MSSSone, data=NSAOHfull, cp = "srs",
             n.chains = 3, burnin = 1000, sample = 1000)

summary(fit1, standardized=TRUE)
fitm1 <- fitMeasures(fit1)
bfit1 <- blavFitIndices(fit1, baseline.model = fit0, rescale="devM", pD="dic")
blavInspect(fit1, what="psrf") #The Gelman-Rubin convergence statistic
mean(bfit1@details$chisq)
mean(bfit1@details$df)
mean(bfit1@indices$BRMSEA)
mean(bfit1@indices$BCFI)
mean(bfit1@details$pD)
HDInterval::hdi(bfit1@indices$BRMSEA, credMass=0.90)
HDInterval::hdi(bfit1@indices$BCFI, credMass=0.90)
bcomp31 <- blavCompare(fit3, fit1) 
BF31 <- exp(mpfr(7851.294, 10000, scientific=TRUE)) 
BF31scientific <- formatMpfr(BF31, scientific=TRUE, digits=3) 

###Calculates 95% CrI for standardized parameters###

stpost1 <- standardizedPosterior(fit1)
stpost1 <- as.data.frame(stpost1)
HDInterval::hdi(stpost1[,1], credMass=0.95)
HDInterval::hdi(stpost1[,2], credMass=0.95)
HDInterval::hdi(stpost1[,3], credMass=0.95)
HDInterval::hdi(stpost1[,4], credMass=0.95)
HDInterval::hdi(stpost1[,5], credMass=0.95)
HDInterval::hdi(stpost1[,6], credMass=0.95)
HDInterval::hdi(stpost1[,7], credMass=0.95)
HDInterval::hdi(stpost1[,8], credMass=0.95)
HDInterval::hdi(stpost1[,9], credMass=0.95)
HDInterval::hdi(stpost1[,10], credMass=0.95)
HDInterval::hdi(stpost1[,11], credMass=0.95)
HDInterval::hdi(stpost1[,12], credMass=0.95)

#Calculation of MCMC standard error#

stpost1 <- standardizedPosterior(fit1)
stpost1 <- as.data.frame(stpost1)
se1 <- NA
for (i in 1:ncol(stpost1)) {
  se1[i] <- mcmcse::mcse(stpost1[,i])[2]
}
se1 <- unlist(se1, use.names=FALSE)
checkvar1 <- NA
for (i in 1:length(se1)) {
  checkvar1[i] <- se1[i]<0.05*sd(stpost1[,i])}
scales::scientific(min(se1[se1 > 0]), digits=3)
scales::scientific(max(se1), digits=3)

## two-factor model ##

fit2 <- bcfa(MSSStwo, data=NSAOHfull, cp = "srs",
             n.chains = 3, burnin = 1000, sample = 1000)
summary(fit2, standardized=TRUE)
blavInspect(fit2, what="psrf")
fitm2 <- fitMeasures(fit2)
bfit2 <- blavFitIndices(fit2, baseline.model = fit0, rescale="devM", pD="dic")
mean(bfit2@details$chisq)
mean(bfit2@details$df)
mean(bfit2@details$pD)
mean(bfit2@indices$BRMSEA)
mean(bfit2@indices$BCFI)
HDInterval::hdi(bfit2@indices$BRMSEA, credMass=0.90)
HDInterval::hdi(bfit2@indices$BCFI, credMass=0.90)
bcomp32 <- blavCompare(fit3, fit2)
BF32 <- exp(mpfr(4618.904, 1000, scientific=TRUE))
BF32scientific <- formatMpfr(BF32, scientific=TRUE, digits=3)

###Calculates 95% CrI for standardized parameters###

stpost2 <- standardizedPosterior(fit2)
stpost2 <- as.data.frame(stpost2)
HDInterval::hdi(stpost2[,1], credMass=0.95)
HDInterval::hdi(stpost2[,2], credMass=0.95)
HDInterval::hdi(stpost2[,3], credMass=0.95)
HDInterval::hdi(stpost2[,4], credMass=0.95)
HDInterval::hdi(stpost2[,5], credMass=0.95)
HDInterval::hdi(stpost2[,6], credMass=0.95)
HDInterval::hdi(stpost2[,7], credMass=0.95)
HDInterval::hdi(stpost2[,8], credMass=0.95)
HDInterval::hdi(stpost2[,9], credMass=0.95)
HDInterval::hdi(stpost2[,10], credMass=0.95)
HDInterval::hdi(stpost2[,11], credMass=0.95)
HDInterval::hdi(stpost2[,12], credMass=0.95)
HDInterval::hdi(stpost2[,27], credMass=0.95)

#Calculation of MCMC standard error#

stpost2 <- standardizedPosterior(fit2)
stpost2 <- as.data.frame(stpost2)
se2 <- NA
for (i in 1:ncol(stpost2)) {
  se2[i] <- mcmcse::mcse(stpost2[,i])[2]
}
se2 <- unlist(se2, use.names=FALSE)
checkvar2 <- NA
for (i in 1:length(se2)) {
  checkvar2[i] <- se2[i]<0.05*sd(stpost2[,i])}
scales::scientific(min(se2[se2 > 0]), digits=3)
scales::scientific(max(se2), digits=3)

### 3-factormodel

fit3 <- bcfa(MSSSthree, data=NSAOHfull, cp = "srs",
             n.chains = 3, burnin = 1000, sample = 1000)
summary(fit3, standardized=TRUE)
blavInspect(fit3, what="psrf") 
blavInspect(fit3, what="neff") 
prior <- blavInspect(fit3, what="dp") 
fitm3 <- fitMeasures(fit3)
bfit3 <- blavFitIndices(fit3, baseline.model = fit0, rescale="devM", pD="dic")
mean(bfit3@details$chisq)
mean(bfit3@details$pD)
mean(bfit3@details$df)
mean(bfit3@indices$BRMSEA)
mean(bfit3@indices$BCFI)
HDInterval::hdi(bfit3@indices$BRMSEA, credMass=0.90)
HDInterval::hdi(bfit3@indices$BCFI, credMass=0.90)

###Calculates 95% CrI for standardized parameters###

stpost3 <- standardizedPosterior(fit3)
stpost3 <- as.data.frame(stpost3)
HDInterval::hdi(stpost3[,1], credMass=0.95)
HDInterval::hdi(stpost3[,2], credMass=0.95)
HDInterval::hdi(stpost3[,3], credMass=0.95)
HDInterval::hdi(stpost3[,4], credMass=0.95)
HDInterval::hdi(stpost3[,5], credMass=0.95)
HDInterval::hdi(stpost3[,6], credMass=0.95)
HDInterval::hdi(stpost3[,7], credMass=0.95)
HDInterval::hdi(stpost3[,8], credMass=0.95)
HDInterval::hdi(stpost3[,9], credMass=0.95)
HDInterval::hdi(stpost3[,10], credMass=0.95)
HDInterval::hdi(stpost3[,11], credMass=0.95)
HDInterval::hdi(stpost3[,12], credMass=0.95)
HDInterval::hdi(stpost3[,28], credMass=0.95)
HDInterval::hdi(stpost3[,29], credMass=0.95)
HDInterval::hdi(stpost3[,30], credMass=0.95)

###Evaluates convergence of MCMC###

#Calculation of MCMC standard error#

se3 <- NA
for (i in 1:ncol(stpost3)) {
  se3[i] <- mcmcse::mcse(stpost3[,i])[2]
}
se3 <- unlist(se3, use.names=FALSE)
checkvar3 <- NA
for (i in 1:length(se3)) {
  checkvar3[i] <- se3[i]<0.05*sd(stpost3[,i])}
scales::scientific(min(se3[se3 > 0]), digits=3)
scales::scientific(max(se3), digits=3)

#Creates Trace Plots#

names(fit3@external$mcmcout) <- c("Loading2", "Loading5","Loading10", "Loading3","Loading4","Loading8",
                                          "Loading6", "Loading7", "Loading9", 
                                          "Var1", "Var2", "Var5", "Var10", "Var3", "Var4", "Var8", "Var11", "Var6",
                                           "Var7", "Var9", "Var12", "LVar1", "LVar2", "LVar3", "Cov1", "Cov2", "Cov3", "Int1", "Int2", "Int5", "Int10",
                                           "Int3", "Int4", "Int8", "Int11", "Int6", "Int7", "Int9", "Int12", "lp__")
bayesplot_theme_set(theme_minimal())
color_scheme_set("mix-blue-red")
png("Fig1SUP.png", res=300, height=5000, width=5000)
bayesplot::mcmc_trace(fit3@external$mcmcout)
dev.off()

###Plots for Posterior distribution of Parameters###

MCMCbinded <- standardizedPosterior(fit3)
MCMCbinded <- as.data.frame(MCMCbinded)
MCMCbinded <- MCMCbinded[,1:12]
names(MCMCbinded) <- c("M1", "M2", "M3", "M4", "M5", "M6", "M7", "M8", "M9", "M10", "M11", "M12")

brmseavalues <- bfit3@indices[["BRMSEA"]]
bcfivalues <- bfit3@indices[["BCFI"]]
afivalues <- cbind(brmseavalues, bcfivalues)
afivalues <- as.data.frame(afivalues)

ggplot(data=MCMCbinded, aes(x=MCMCbinded$`two=~M11`))  + 
  labs(y = "Density", x = "Item 11") +
  geom_density(color="lightskyblue4", fill="lightskyblue", alpha=0.8) 

ggplot(data=MCMCbinded, aes(x=MCMCbinded$`two~~three`))  + 
  labs(y = "Density", x = "Family ~ Friends correlation") +
  geom_density(color="khaki4", fill="khaki1", alpha=0.8) 

png("PosteriorFIT.png", res=300, height=5000, width=5000)
plot1 <- ggplot(data=afivalues, aes(x=afivalues$brmseavalues))  + 
  labs(y = "Density", x = "BRMSEA") +
  theme_minimal() +
  geom_density(color="darkred", fill="#F8766D") 
plot2 <- ggplot(data=afivalues, aes(x=afivalues$bcfivalues))  + 
  labs(y = "Density", x = "BCFI") +
  theme_minimal() +
  geom_density(color="khaki4", fill="khaki1") 
grid.arrange(plot1, plot2, ncol=2, nrow=2)
dev.off()

### calculates McDonald's Omega Bayesian ###

###Posterior mean of parameters###
paramb <- blavInspect(fit3, "mcmc")
#Get the factor loadings across all iterations

for (i in 1:9) {
  assign(paste("parf", i, sep = ""), rbind(paramb[[1]][,i], paramb[[2]][,i],  paramb[[3]][,i]))
}

parf1 <- as.vector(parf1)
parf2 <- as.vector(parf2)
parf3 <- as.vector(parf3)
parf4 <- as.vector(parf4)
parf5 <- as.vector(parf5)
parf6 <- as.vector(parf6)
parf7 <- as.vector(parf7)
parf8 <- as.vector(parf8)
parf9 <- as.vector(parf9)

#Get the factor variances across all iterations

for (i in 22:24) {
  assign(paste("varf", i-21, sep = ""), rbind(paramb[[1]][,i], paramb[[2]][,i],  paramb[[3]][,i]))
}

#Sum and square the factor loadings
allpar1 <- cbind(parf1,parf2,parf3)
allpar2 <- cbind(parf4,parf5,parf6)
allpar3 <- cbind(parf7,parf8,parf9)
vecsumd1 <- rowSums(allpar1)+1
vecsumd2 <- rowSums(allpar2)+1
vecsumd3 <- rowSums(allpar3)+1
vecsqrd1 <- vecsumd1*vecsumd1
vecsqrd2 <- vecsumd2*vecsumd2
vecsqrd3 <- vecsumd3*vecsumd3
#Calculates the Bayesian omega nominator
nom1 <- vecsqrd1 * varf1
nom2 <- vecsqrd2 * varf2
nom3 <- vecsqrd3 * varf3
#Extract the observed covariance matrix and calculates the denominator
S <- lavInspect(fit_ml3, "sampstat")$cov
den1 <- S[1:4,1:4]
den1 <- sum(den1)
den1 <- rep(den1, times=length(nom1))
den2 <- S[5:8,5:8]
den2 <- sum(den2)
den2 <- rep(den2, times=length(nom2))
den3 <- S[9:12,9:12]
den3 <- sum(den3)
den3 <- rep(den3, times=length(nom3))
#Calculates the Bayesian omega
bomegadist1 <- nom1/den1
bomegadist1 <- as.vector(bomegadist1)
bomegadist2 <- nom2/den2
bomegadist2 <- as.vector(bomegadist2)
bomegadist3 <- nom3/den3 
bomegadist3 <- as.vector(bomegadist3)
paste(mean(bomegadist1), mean(bomegadist2), mean(bomegadist3))
HDInterval::hdi(bomegadist1, credMass=0.95)
HDInterval::hdi(bomegadist2, credMass=0.95)
HDInterval::hdi(bomegadist3, credMass=0.95)
reliability(fit_ml3)
